package hdl

import (
	"applet/app/e"
	"applet/app/lib/validate"
	"applet/app/md"
	"applet/app/svc"
	"github.com/gin-gonic/gin"
)

// SettleCenterMediumList
// @Summary    	 媒体结算报表
// @Tags         结算中心------嘉俊
// @Description  结算中心-媒体结算报表
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Accept       json
// @Produce      json
// @Param        args 	body      md.SettleCenterDataReq   true   "请求参数"
// @Success      200   {object}	  md.SettleCenterMediumDataRes     "具体看返回内容 这是data里面的数据"
// @Failure      400   {object}   md.Response              "具体错误"
// @Router       /api/settleCenter/medium/list [POST]
func SettleCenterMediumList(c *gin.Context) {
	var req md.SettleCenterDataReq
	err := c.ShouldBindJSON(&req)
	if err != nil {
		err = validate.HandleValidateErr(err)
		err1 := err.(e.E)
		e.OutErr(c, err1.Code, err1.Error())
		return
	}
	res := svc.SettleCenterMediumList(c, req)
	e.OutSuc(c, res, nil)
	return
}

// SettleCenterList
// @Summary    	 渠道个人结算报表
// @Tags         结算中心------嘉俊
// @Description  结算中心-渠道个人结算报表
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Accept       json
// @Produce      json
// @Param        args 	body      md.SettleCenterDataReq   true   "请求参数"
// @Success      200   {object}	  md.SettleCenterDataRes     "具体看返回内容 这是data里面的数据"
// @Failure      400   {object}   md.Response              "具体错误"
// @Router       /api/settleCenter/agent/list [POST]
func SettleCenterList(c *gin.Context) {
	var req md.SettleCenterDataReq
	err := c.ShouldBindJSON(&req)
	if err != nil {
		err = validate.HandleValidateErr(err)
		err1 := err.(e.E)
		e.OutErr(c, err1.Code, err1.Error())
		return
	}
	res := svc.SettleCenterList(c, req)
	e.OutSuc(c, res, nil)
	return
}

// SettleCenterDetail
// @Summary    	 渠道个人结算报表-详情
// @Tags         结算中心------嘉俊
// @Description  结算中心-渠道个人结算报表-详情
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Accept       json
// @Produce      json
// @Param        args 	body      md.CommDetailReq   true   "请求参数"
// @Success      200   {object}	   md.SettleCenterDataDetailRes   "具体看返回内容 这是data里面的数据"
// @Failure      400   {object}   md.Response              "具体错误"
// @Router       /api/settleCenter/agent/detail [POST]
func SettleCenterDetail(c *gin.Context) {
	var req md.CommDetailReq
	err := c.ShouldBindJSON(&req)
	if err != nil {
		err = validate.HandleValidateErr(err)
		err1 := err.(e.E)
		e.OutErr(c, err1.Code, err1.Error())
		return
	}
	res := svc.SettleCenterDetail(c, req)
	e.OutSuc(c, res, nil)
	return
}

// SettleCenterSettleFileSave
// @Summary    	渠道个人详情-结算单保存
// @Tags         财务中心------嘉俊
// @Description  财务中心-渠道个人详情-结算单保存
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Accept       json
// @Produce      json
// @Param        args 	body      md.SettleFileReq   true   "请求参数"
// @Success      200   {string}	     "具体看返回内容 这是data里面的数据"
// @Failure      400   {object}   md.Response              "具体错误"
// @Router       /api/settleCenter/agent/settle/file/save [POST]
func SettleCenterSettleFileSave(c *gin.Context) {
	var req md.SettleFileReq
	err := c.ShouldBindJSON(&req)
	if err != nil {
		err = validate.HandleValidateErr(err)
		err1 := err.(e.E)
		e.OutErr(c, err1.Code, err1.Error())
		return
	}
	svc.SettleCenterSettleFileSave(c, req)
}

// SettleCenterInvoiceSave
// @Summary    	渠道个人详情-发票上传
// @Tags         财务中心------嘉俊
// @Description  财务中心-渠道个人详情-发票上传
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Accept       json
// @Produce      json
// @Param        args 	body      md.InvoiceReq   true   "请求参数"
// @Success      200   {string}	     "具体看返回内容 这是data里面的数据"
// @Failure      400   {object}   md.Response              "具体错误"
// @Router       /api/settleCenter/agent/invoice/save [POST]
func SettleCenterInvoiceSave(c *gin.Context) {
	var req md.InvoiceReq
	err := c.ShouldBindJSON(&req)
	if err != nil {
		err = validate.HandleValidateErr(err)
		err1 := err.(e.E)
		e.OutErr(c, err1.Code, err1.Error())
		return
	}
	svc.SettleCenterInvoiceSave(c, req)
}
