package router

import (
	"applet/app/cfg"
	"applet/app/hdl"
	"applet/app/mw"
	_ "applet/docs"
	"github.com/gin-gonic/gin"
	swaggerFiles "github.com/swaggo/files"
	ginSwagger "github.com/swaggo/gin-swagger"
)

// 初始化路由
func Init() *gin.Engine {
	// debug, release, test 项目阶段
	mode := "release"
	if cfg.Debug {
		mode = "debug"
	}
	gin.SetMode(mode)
	//创建一个新的启动器
	r := gin.New()
	r.GET("/api/swagger/*any", func(c *gin.Context) {
		//r.Use(mw.SwagAuth())
		ginSwagger.DisablingWrapHandler(swaggerFiles.Handler, "SWAGGER")(c)
	})

	r.Use(mw.ChangeHeader)

	// 是否打印访问日志, 在非正式环境都打印
	if mode != "release" {
		r.Use(gin.Logger())
	}
	r.Use(gin.Recovery())
	// r.Use(mw.Limiter)
	//r.LoadHTMLGlob("static/html/*")

	r.GET("/favicon.ico", func(c *gin.Context) {
		c.Status(204)
	})
	r.NoRoute(func(c *gin.Context) {
		c.JSON(404, gin.H{"code": 404, "msg": "page not found", "data": []struct{}{}})
	})
	r.NoMethod(func(c *gin.Context) {
		c.JSON(405, gin.H{"code": 405, "msg": "method not allowed", "data": []struct{}{}})
	})
	r.Use(mw.Cors)
	route(r.Group("/api"))
	return r
}

func route(r *gin.RouterGroup) {
	r.GET("/test", hdl.Demo)
	r.POST("/qiniuyun/callback", hdl.FileImgCallback) //七牛云回调
	r.Use(mw.DB)                                      // 以下接口需要用到数据库
	{
		r.POST("/login", hdl.Login)
		r.POST("/login/phone", hdl.LoginPhone)
		r.POST("/register", hdl.Register)
	}
	r.Use(mw.CheckBody) //body参数转换
	r.Use(mw.CheckSign) //签名校验
	{
		r.POST("/qiniuyun/upload", hdl.ImgReqUpload) //七牛云上传
		r.POST("/sms", hdl.Sms)                      //短信发送
	}
	r.Use(mw.Auth)                          // 以下接口需要JWT验证
	rRole(r.Group("/role"))                 //权限管理
	rMedium(r.Group("/medium"))             //媒体中心
	rAccount(r.Group("/account"))           //账号中心
	rIndex(r.Group("/index"))               //首页
	rDataCenter(r.Group("/dataCenter"))     //数据中心
	rSettleCenter(r.Group("/settleCenter")) //结算中心

}

func rRole(r *gin.RouterGroup) {
	r.GET("/roleList", hdl.RoleList)                                //角色列表
	r.POST("/addRole", hdl.AddRole)                                 //角色添加
	r.POST("/roleBindPermissionGroup", hdl.RoleBindPermissionGroup) //角色绑定权限组
	r.POST("/updateRoleState", hdl.UpdateRoleState)                 //修改角色状态
	r.POST("/updateRole", hdl.UpdateRole)                           //修改角色状态
	r.DELETE("/deleteRole/:id", hdl.DeleteRole)                     //删除角色
	r.GET("/permissionGroupList", hdl.PermissionGroupList)          //权限组列表
	r.POST("/adminList", hdl.AdminList)                             //管理员列表
	r.POST("/updateAdminState", hdl.UpdateAdminState)               //修改管理员状态
	r.POST("/updateAdmin", hdl.UpdateAdmin)                         //修改管理员信息
	r.POST("/addAdmin", hdl.AddAdmin)                               //新增管理员
	r.DELETE("/deleteAdmin/:adm_id", hdl.DeleteAdmin)               //删除管理员
	r.GET("/adminInfo", hdl.AdminInfo)                              //获取管理员信息
	r.POST("/bindAdminRole", hdl.BindAdminRole)                     //绑定角色
}
func rMedium(r *gin.RouterGroup) {
	r.POST("/list", hdl.MediumList) //媒体列表
}
func rAccount(r *gin.RouterGroup) {
	r.GET("/base", hdl.AccountBase)                                           //账号信息
	r.POST("/update/password", hdl.AccountUpdatePassword)                     //账号信息-修改密码
	r.GET("/qualification/info", hdl.QualificationInfo)                       //资质认证-基本信息
	r.GET("/qualification/select/base", hdl.QualificationSelectBase)          //资质认证-认证选择内容
	r.POST("/qualification/enterprise/save", hdl.QualificationEnterpriseSave) //资质认证-企业认证保存
	r.POST("/qualification/bank/save", hdl.QualificationBankSave)             //资质认证-银行资质保存
	r.POST("/qualification/contact/save", hdl.QualificationContactSave)       //资质认证-联系方式保存
}
func rIndex(r *gin.RouterGroup) {
	r.GET("/base", hdl.Base)              //首页-基本信息
	r.GET("/total", hdl.IndexTotal)       //首页-统计数据
	r.POST("/app/list", hdl.IndexAppList) //首页-应用数据
}
func rDataCenter(r *gin.RouterGroup) {
	r.GET("/select/data", hdl.DataCenterSelectData)      //数据中心-筛选条件
	r.POST("/record/total", hdl.DataCenterRecordTotal)   //数据中心-数据明细合计
	r.POST("/record/list", hdl.DataCenterRecordList)     //数据中心-数据明细
	r.POST("/record/output", hdl.DataCenterRecordOutPut) //数据中心-数据明细-导出

	r.POST("/commission/record/total", hdl.DataCenterCommissionRecordTotal)   //数据中心-佣金收益明细合计
	r.POST("/commission/record/list", hdl.DataCenterCommissionRecordList)     //数据中心-佣金收益明细
	r.POST("/commission/record/output", hdl.DataCenterCommissionRecordOutPut) //数据中心-佣金收益明细-导出

	r.POST("/profit/record/list", hdl.DataCenterProfitRecordList)     //数据中心-分润收益明细
	r.POST("/profit/record/output", hdl.DataCenterProfitRecordOutPut) //数据中心-分润收益明细-导出

}
func rSettleCenter(r *gin.RouterGroup) {
	r.POST("/medium/list", hdl.SettleCenterMediumList)                //结算中心-媒体结算报表
	r.POST("/agent/list", hdl.SettleCenterList)                       //结算中心-渠道个人结算报表
	r.POST("/agent/detail", hdl.SettleCenterDetail)                   //结算中心-渠道个人结算报表-详情
	r.POST("/agent/settle/file/save", hdl.SettleCenterSettleFileSave) //结算中心-渠道个人结算报表-结算单上传
	r.POST("/agent/invoice/save", hdl.SettleCenterInvoiceSave)        //结算中心-渠道个人结算报表-发票上传
}
